@@ -22,38 +22,44 @@
 # ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 # CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 # SOFTWARE.
-
-
 import hashlib
-import os.path
-import re
 import sys
-import threading
 import time
 import traceback
-import urlparse
 import json
 import requests
 
+import urllib.parse
+
+
 try:
-    import paymentrequest_pb2 as pb2
+    from . import paymentrequest_pb2 as pb2
 except ImportError:
     sys.exit("Error: could not find paymentrequest_pb2.py. Create it with 'protoc --proto_path=lib/ --python_out=lib/ lib/paymentrequest.proto'")
 
-import bitcoin
-import util
-from util import print_error
-import transaction
-import x509
-import rsakey
+from . import bitcoin
+from . import ecc
+from . import util
+from .util import print_error, bh2u, bfh
+from .util import export_meta, import_meta
+from . import transaction
+from . import x509
+from . import rsakey
 
-from bitcoin import TYPE_ADDRESS
+from .bitcoin import TYPE_ADDRESS
 
 REQUEST_HEADERS = {'Accept': 'application/bitcoin-paymentrequest', 'User-Agent': 'Electrum'}
 ACK_HEADERS = {'Content-Type':'application/bitcoin-payment','Accept':'application/bitcoin-paymentack','User-Agent':'Electrum'}
 
 ca_path = requests.certs.where()
-ca_list, ca_keyID = x509.load_certificates(ca_path)
+ca_list = None
+ca_keyID = None
+
+def load_ca_list():
+    global ca_list, ca_keyID
+    if ca_list is None:
+        ca_list, ca_keyID = x509.load_certificates(ca_path)
+
 
 
 # status of payment requests
@@ -65,7 +71,7 @@ PR_PAID    = 3     # send and propagated
 
 
 def get_payment_request(url):
-    u = urlparse.urlparse(url)
+    u = urllib.parse.urlparse(url)
     error = None
     if u.scheme in ['http', 'https']:
         try:
@@ -84,13 +90,13 @@ def get_payment_request(url):
             error = "payment URL not pointing to a valid server"
     elif u.scheme == 'file':
         try:
-            with open(u.path, 'r') as f:
+            with open(u.path, 'r', encoding='utf-8') as f:
                 data = f.read()
         except IOError:
             data = None
             error = "payment URL not pointing to a valid file"
     else:
-        raise BaseException("unknown scheme", url)
+        raise Exception("unknown scheme", url)
     pr = PaymentRequest(data, error)
     return pr
 
@@ -110,7 +116,7 @@ class PaymentRequest:
     def parse(self, r):
         if self.error:
             return
-        self.id = bitcoin.sha256(r)[0:16].encode('hex')
+        self.id = bh2u(bitcoin.sha256(r)[0:16])
         try:
             self.data = pb2.PaymentRequest()
             self.data.ParseFromString(r)
@@ -143,7 +149,7 @@ class PaymentRequest:
             self.error = "Error: Cannot parse payment request"
             return False
         if not pr.signature:
-            # the address will be dispayed as requestor
+            # the address will be displayed as requestor
             self.requestor = None
             return True
         if pr.pki_type in ["x509+sha256", "x509+sha1"]:
@@ -155,6 +161,7 @@ class PaymentRequest:
             return False
 
     def verify_x509(self, paymntreq):
+        load_ca_list()
         if not ca_list:
             self.error = "Trusted certificate authorities list not found"
             return False
@@ -174,7 +181,7 @@ class PaymentRequest:
         # verify the BIP70 signature
         pubkey0 = rsakey.RSAKey(x.modulus, x.exponent)
         sig = paymntreq.signature
-        paymntreq.signature = ''
+        paymntreq.signature = b''
         s = paymntreq.SerializeToString()
         sigBytes = bytearray(sig)
         msgBytes = bytearray(s)
@@ -200,9 +207,9 @@ class PaymentRequest:
         if pr.pki_type == "dnssec+btc":
             self.requestor = alias
             address = info.get('address')
-            pr.signature = ''
+            pr.signature = b''
             message = pr.SerializeToString()
-            if bitcoin.verify_message(address, sig, message):
+            if ecc.verify_message_with_address(address, sig, message):
                 self.error = 'Verified with DNSSEC'
                 return True
             else:
@@ -253,49 +260,39 @@ class PaymentRequest:
         return self.outputs[:]
 
     def send_ack(self, raw_tx, refund_addr):
-
         pay_det = self.details
         if not self.details.payment_url:
             return False, "no url"
-
         paymnt = pb2.Payment()
         paymnt.merchant_data = pay_det.merchant_data
-        paymnt.transactions.append(raw_tx)
-
+        paymnt.transactions.append(bfh(raw_tx))
         ref_out = paymnt.refund_to.add()
-        ref_out.script = transaction.Transaction.pay_script(TYPE_ADDRESS, refund_addr)
+        ref_out.script = util.bfh(transaction.Transaction.pay_script(TYPE_ADDRESS, refund_addr))
         paymnt.memo = "Paid using Electrum"
         pm = paymnt.SerializeToString()
-
-        payurl = urlparse.urlparse(pay_det.payment_url)
+        payurl = urllib.parse.urlparse(pay_det.payment_url)
         try:
             r = requests.post(payurl.geturl(), data=pm, headers=ACK_HEADERS, verify=ca_path)
         except requests.exceptions.SSLError:
-            print "Payment Message/PaymentACK verify Failed"
+            print("Payment Message/PaymentACK verify Failed")
             try:
                 r = requests.post(payurl.geturl(), data=pm, headers=ACK_HEADERS, verify=False)
             except Exception as e:
-                print e
+                print(e)
                 return False, "Payment Message/PaymentACK Failed"
-
         if r.status_code >= 500:
             return False, r.reason
-
         try:
             paymntack = pb2.PaymentACK()
             paymntack.ParseFromString(r.content)
         except Exception:
             return False, "PaymentACK could not be processed. Payment was sent; please manually verify that payment was received."
-
-        print "PaymentACK message received: %s" % paymntack.memo
+        print("PaymentACK message received: %s" % paymntack.memo)
         return True, paymntack.memo
 
-    def set_paid(self, tx_hash):
-        self.tx = tx_hash
-
 
 def make_unsigned_request(req):
-    from transaction import Transaction
+    from .transaction import Transaction
     addr = req['address']
     time = req.get('time', 0)
     exp = req.get('exp', 0)
@@ -307,7 +304,7 @@ def make_unsigned_request(req):
     if amount is None:
         amount = 0
     memo = req['memo']
-    script = Transaction.pay_script(TYPE_ADDRESS, addr).decode('hex')
+    script = bfh(Transaction.pay_script(TYPE_ADDRESS, addr))
     outputs = [(script, amount)]
     pd = pb2.PaymentDetails()
     for script, amount in outputs:
@@ -317,7 +314,7 @@ def make_unsigned_request(req):
     pd.memo = memo
     pr = pb2.PaymentRequest()
     pr.serialized_payment_details = pd.SerializeToString()
-    pr.signature = ''
+    pr.signature = util.to_bytes('')
     return pr
 
 
@@ -325,15 +322,14 @@ def sign_request_with_alias(pr, alias, alias_privkey):
     pr.pki_type = 'dnssec+btc'
     pr.pki_data = str(alias)
     message = pr.SerializeToString()
-    ec_key = bitcoin.regenerate_key(alias_privkey)
-    address = bitcoin.address_from_private_key(alias_privkey)
+    ec_key = ecc.ECPrivkey(alias_privkey)
     compressed = bitcoin.is_compressed(alias_privkey)
-    pr.signature = ec_key.sign_message(message, compressed, address)
-
+    pr.signature = ec_key.sign_message(message, compressed)
 
 
 def verify_cert_chain(chain):
     """ Verify a chain of certificates. The last certificate is the CA"""
+    load_ca_list()
     # parse the chain
     cert_num = len(chain)
     x509_chain = []
@@ -344,9 +340,9 @@ def verify_cert_chain(chain):
             x.check_date()
         else:
             if not x.check_ca():
-                raise BaseException("ERROR: Supplied CA Certificate Error")
+                raise Exception("ERROR: Supplied CA Certificate Error")
     if not cert_num > 1:
-        raise BaseException("ERROR: CA Certificate Chain Not Provided by Payment Processor")
+        raise Exception("ERROR: CA Certificate Chain Not Provided by Payment Processor")
     # if the root CA is not supplied, add it to the chain
     ca = x509_chain[cert_num-1]
     if ca.getFingerprint() not in ca_list:
@@ -356,7 +352,7 @@ def verify_cert_chain(chain):
             root = ca_list[f]
             x509_chain.append(root)
         else:
-            raise BaseException("Supplied CA Not Found in Trusted CA Store.")
+            raise Exception("Supplied CA Not Found in Trusted CA Store.")
     # verify the chain of signatures
     cert_num = len(x509_chain)
     for i in range(1, cert_num):
@@ -377,21 +373,21 @@ def verify_cert_chain(chain):
             hashBytes = bytearray(hashlib.sha512(data).digest())
             verify = pubkey.verify(sig, x509.PREFIX_RSA_SHA512 + hashBytes)
         else:
-            raise BaseException("Algorithm not supported")
+            raise Exception("Algorithm not supported")
             util.print_error(self.error, algo.getComponentByName('algorithm'))
         if not verify:
-            raise BaseException("Certificate not Signed by Provided CA Certificate Chain")
+            raise Exception("Certificate not Signed by Provided CA Certificate Chain")
 
     return x509_chain[0], ca
 
 
 def check_ssl_config(config):
-    import pem
+    from . import pem
     key_path = config.get('ssl_privkey')
     cert_path = config.get('ssl_chain')
-    with open(key_path, 'r') as f:
+    with open(key_path, 'r', encoding='utf-8') as f:
         params = pem.parse_private_key(f.read())
-    with open(cert_path, 'r') as f:
+    with open(cert_path, 'r', encoding='utf-8') as f:
         s = f.read()
     bList = pem.dePemList(s, "CERTIFICATE")
     # verify chain
@@ -408,15 +404,15 @@ def check_ssl_config(config):
     return requestor
 
 def sign_request_with_x509(pr, key_path, cert_path):
-    import pem
-    with open(key_path, 'r') as f:
+    from . import pem
+    with open(key_path, 'r', encoding='utf-8') as f:
         params = pem.parse_private_key(f.read())
         privkey = rsakey.RSAKey(*params)
-    with open(cert_path, 'r') as f:
+    with open(cert_path, 'r', encoding='utf-8') as f:
         s = f.read()
         bList = pem.dePemList(s, "CERTIFICATE")
     certificates = pb2.X509Certificates()
-    certificates.certificate.extend(map(str, bList))
+    certificates.certificate.extend(map(bytes, bList))
     pr.pki_type = 'x509+sha256'
     pr.pki_data = certificates.SerializeToString()
     msgBytes = bytearray(pr.SerializeToString())
@@ -430,7 +426,7 @@ def serialize_request(req):
     signature = req.get('sig')
     requestor = req.get('name')
     if requestor and signature:
-        pr.signature = signature.decode('hex')
+        pr.signature = bfh(signature)
         pr.pki_type = 'dnssec+btc'
         pr.pki_data = str(requestor)
     return pr
@@ -448,42 +444,63 @@ def make_request(config, req):
 
 class InvoiceStore(object):
 
-    def __init__(self, config):
-        self.config = config
+    def __init__(self, storage):
+        self.storage = storage
         self.invoices = {}
-        self.load_invoices()
+        self.paid = {}
+        d = self.storage.get('invoices', {})
+        self.load(d)
 
-    def load_invoices(self):
-        path = os.path.join(self.config.path, 'invoices')
-        try:
-            with open(path, 'r') as f:
-                d = json.loads(f.read())
-        except:
-            return
+    def set_paid(self, pr, txid):
+        pr.tx = txid
+        pr_id = pr.get_id()
+        self.paid[txid] = pr_id
+        if pr_id not in self.invoices:
+            # in case the user had deleted it previously
+            self.add(pr)
+
+    def load(self, d):
         for k, v in d.items():
             try:
-                pr = PaymentRequest(v.get('hex').decode('hex'))
+                pr = PaymentRequest(bfh(v.get('hex')))
                 pr.tx = v.get('txid')
                 pr.requestor = v.get('requestor')
                 self.invoices[k] = pr
+                if pr.tx:
+                    self.paid[pr.tx] = k
             except:
                 continue
 
-    def save(self):
-        l = {}
+    def import_file(self, path):
+        def validate(data):
+            return data  # TODO
+        import_meta(path, validate, self.on_import)
+
+    def on_import(self, data):
+        self.load(data)
+        self.save()
+
+    def export_file(self, filename):
+        export_meta(self.dump(), filename)
+
+    def dump(self):
+        d = {}
         for k, pr in self.invoices.items():
-            l[k] = {
-                'hex': str(pr).encode('hex'),
+            d[k] = {
+                'hex': bh2u(pr.raw),
                 'requestor': pr.requestor,
                 'txid': pr.tx
             }
-        path = os.path.join(self.config.path, 'invoices')
-        with open(path, 'w') as f:
-            s = json.dumps(l, indent=4, sort_keys=True)
-            r = f.write(s)
+        return d
+
+    def save(self):
+        self.storage.put('invoices', self.dump())
 
     def get_status(self, key):
         pr = self.get(key)
+        if pr is None:
+            print_error("[InvoiceStore] get_status() can't find pr for", key)
+            return
         if pr.tx is not None:
             return PR_PAID
         if pr.has_expired():
@@ -507,3 +524,5 @@ class InvoiceStore(object):
         # sort
         return self.invoices.values()
 
+    def unpaid_invoices(self):
+        return [ self.invoices[k] for k in filter(lambda x: self.get_status(x)!=PR_PAID, self.invoices.keys())]
