@@ -1,13 +1,53 @@
 import unittest
 from lib.util import format_satoshis, parse_URI
 
-class TestUtil(unittest.TestCase):
+from . import SequentialTestCase
+
+
+class TestUtil(SequentialTestCase):
 
     def test_format_satoshis(self):
         result = format_satoshis(1234)
         expected = "0.00001234"
         self.assertEqual(expected, result)
 
+    def test_format_satoshis_negative(self):
+        result = format_satoshis(-1234)
+        expected = "-0.00001234"
+        self.assertEqual(expected, result)
+
+    def test_format_fee(self):
+        result = format_satoshis(1700/1000, 0, 0)
+        expected = "1.7"
+        self.assertEqual(expected, result)
+
+    def test_format_fee_precision(self):
+        result = format_satoshis(1666/1000, 0, 0, precision=6)
+        expected = "1.666"
+        self.assertEqual(expected, result)
+
+        result = format_satoshis(1666/1000, 0, 0, precision=1)
+        expected = "1.7"
+        self.assertEqual(expected, result)
+
+    def test_format_satoshis_whitespaces(self):
+        result = format_satoshis(12340, whitespaces=True)
+        expected = "     0.0001234 "
+        self.assertEqual(expected, result)
+
+        result = format_satoshis(1234, whitespaces=True)
+        expected = "     0.00001234"
+        self.assertEqual(expected, result)
+
+    def test_format_satoshis_whitespaces_negative(self):
+        result = format_satoshis(-12340, whitespaces=True)
+        expected = "    -0.0001234 "
+        self.assertEqual(expected, result)
+
+        result = format_satoshis(-1234, whitespaces=True)
+        expected = "    -0.00001234"
+        self.assertEqual(expected, result)
+
     def test_format_satoshis_diff_positive(self):
         result = format_satoshis(1234, is_diff=True)
         expected = "+0.00001234"
@@ -60,11 +100,10 @@ class TestUtil(unittest.TestCase):
                                 {'r': 'http://domain.tld/page?h=2a8628fc2fbe'})
 
     def test_parse_URI_invalid_address(self):
-        self.assertRaises(AssertionError, parse_URI, 'bitcoin:invalidaddress')
+        self.assertRaises(BaseException, parse_URI, 'bitcoin:invalidaddress')
 
     def test_parse_URI_invalid(self):
-        self.assertRaises(AssertionError, parse_URI, 'notbitcoin:15mKKb2eos1hWa6tisdPwwDC1a5J1y9nma')
+        self.assertRaises(BaseException, parse_URI, 'notbitcoin:15mKKb2eos1hWa6tisdPwwDC1a5J1y9nma')
 
     def test_parse_URI_parameter_polution(self):
         self.assertRaises(Exception, parse_URI, 'bitcoin:15mKKb2eos1hWa6tisdPwwDC1a5J1y9nma?amount=0.0003&label=test&amount=30.0')
-
